@@ -1,6 +1,8 @@
 '''The bulk of PIUS - al the magic to do individual UID signing.'''
 
-from libpius.util import debug, clean_files
+# vim:shiftwidth=2:tabstop=2:expandtab:textwidth=80:softtabstop=2:ai:
+
+from libpius.util import debug, clean_files, logcmd
 from libpius.constants import *
 from libpius.exceptions import *
 import getpass
@@ -51,19 +53,22 @@ class PiusSigner(object):
     self.mail_host = mail_host
     self.null = open(os.path.devnull, 'w')
     self.gpg_base_opts = [
-      '--keyid-format', 'long'
+          '--keyid-format', 'long',
+          '--no-auto-check-trustdb',
     ]
     self.gpg_quiet_opts = [
-      '-q',
-      '--no-tty',
-      '--no-auto-check-trustdb',
-      '--batch',
+          '-q',
+          '--no-tty',
+          '--batch',
     ]
     self.gpg_fd_opts = [
-      '--command-fd', '0',
-      '--passphrase-fd', '0',
-      '--status-fd', '1',
+          '--command-fd', '0',
+          '--status-fd', '1',
     ]
+    self.gpg2 = self._is_gpg2()
+
+    if not self.gpg2:
+        self.gpg_fd_opts += ['--passphrase-fd', '0',]
 
     if self.mode == MODE_INTERACTIVE:
       try:
@@ -75,6 +80,35 @@ class PiusSigner(object):
         parser.error('You chose interactive mode but do not have the pexpect'
                      ' module installed.')
 
+  def _is_gpg2(self):
+    cmd = [self.gpg, '--version']
+    logcmd(cmd)
+    gpg = subprocess.Popen(
+        cmd,
+        stdin=self.null,
+        stdout=subprocess.PIPE,
+        stderr=self.null,
+    )
+
+    v = None
+    for line in gpg.stdout:
+      # On Linux this looks like:
+      #   gpg (GnuPG) 2.1.8
+      # On Mac this looks like:
+      #   gpg (GnuPG/MacGPG2) 2.0.28
+      m = re.match(r'^gpg \(GnuPG.*\) ([0-9\.]+)$', line)
+      if m:
+        v = m.group(1)
+
+    if not v:
+      print "ERROR: Could not determine gpg version\n"
+      sys.exit(1)
+
+    return v.startswith('2.')
+
+  def is_gpg2(self):
+    return self.gpg2
+
   def _outfile_path(self, ofile):
     '''Internal function to take a filename and put it in self.outdir.'''
     return '%s/%s' % (self.outdir, ofile)
@@ -91,19 +125,21 @@ class PiusSigner(object):
     '''Given a keyring, get all the KeyIDs from it.'''
     debug('extracting all keyids from keyring')
     cmd = [self.gpg] + self.gpg_base_opts + [
-      '--no-default-keyring',
-      '--keyring', self.keyring,
-      '--no-options',
-      '--with-colons',
-      '--keyid-format', 'long',
-      '--fingerprint',
-      '--fixed-list-mode',
+        '--no-default-keyring',
+        '--keyring', self.keyring,
+        '--no-options',
+        '--with-colons',
+        '--keyid-format', 'long',
+        '--fingerprint',
+        '--fixed-list-mode',
     ]
-    debug(' '.join(cmd))
-    gpg = subprocess.Popen(cmd,
+    logcmd(cmd)
+    gpg = subprocess.Popen(
+        cmd,
         stdin=self.null,
         stdout=subprocess.PIPE,
-        stderr=subprocess.STDOUT)
+        stderr=subprocess.STDOUT
+    )
     # We use 'pub' instead of 'fpr' to support old crufty keys too...
     pub_re = re.compile('^pub:')
     uid_re = re.compile('^uid:')
@@ -122,9 +158,9 @@ class PiusSigner(object):
 
     # sort the list
     if self.sort_keyring:
-      keyids = [ i[1] for i in sorted(key_tuples) ]
+      keyids = [i[1] for i in sorted(key_tuples)]
     else:
-      keyids = [ i[1] for i in key_tuples ]
+      keyids = [i[1] for i in key_tuples]
     return keyids
 
   def _print_cert_levels(self):
@@ -133,11 +169,11 @@ class PiusSigner(object):
   def check_fingerprint(self, key):
     '''Prompt the user to see if they have verified this fingerprint.'''
     cmd = [self.gpg] + self.gpg_base_opts + self.gpg_quiet_opts + [
-      '--no-default-keyring',
-      '--keyring', self.keyring,
-      '--fingerprint', key,
+        '--no-default-keyring',
+        '--keyring', self.keyring,
+        '--fingerprint', key,
     ]
-    debug(cmd)
+    logcmd(cmd)
     gpg = subprocess.Popen(cmd,
                            stdin=self.null,
                            stdout=subprocess.PIPE,
@@ -188,19 +224,19 @@ class PiusSigner(object):
     tfile.write(magic_string)
     tfile.close()
     cmd = [self.gpg] + self.gpg_base_opts + self.gpg_quiet_opts + [
-      '--no-armor',
-      '--always-trust',
-      '-r', self.signer,
-      '-e', filename,
+        '--no-armor',
+        '--always-trust',
+        '-r', self.signer,
+        '-e', filename,
     ]
-    debug(cmd)
+    logcmd(cmd)
     subprocess.call(cmd, stdout=self.null, stderr=self.null, close_fds=True)
     cmd = [self.gpg] + self.gpg_base_opts + self.gpg_quiet_opts + \
       self.gpg_fd_opts + [
-        '--output', filename_dec,
-        '-d', filename_enc,
+          '--output', filename_dec,
+          '-d', filename_enc,
       ]
-    debug(cmd)
+    logcmd(cmd)
     gpg = subprocess.Popen(cmd, stdin=subprocess.PIPE,
                            stdout=subprocess.PIPE,
                            stderr=self.null,
@@ -234,13 +270,13 @@ class PiusSigner(object):
     '''Get all UIDs on a given key.'''
     cmd = [self.gpg] + self.gpg_base_opts +  self.gpg_quiet_opts + \
       self.gpg_fd_opts + [
-        '--no-default-keyring',
-        '--keyring', self.keyring,
-        '--no-options',
-        '--with-colons',
-        '--edit-key', key,
+          '--no-default-keyring',
+          '--keyring', self.keyring,
+          '--no-options',
+          '--with-colons',
+          '--edit-key', key,
       ]
-    debug(cmd)
+    logcmd(cmd)
     gpg = subprocess.Popen(cmd, stdin=subprocess.PIPE,
                            stdout=subprocess.PIPE,
                            stderr=self.null,
@@ -273,7 +309,7 @@ class PiusSigner(object):
 
       debug('Got UID %s with status %s' % (uid, status))
 
-      # If we can we capture an email address is saved for 
+      # If we can we capture an email address is saved for
       # emailing off signed keys (not yet implemented), and
       # also for the ID for that UID.
       #
@@ -351,15 +387,15 @@ class PiusSigner(object):
       os.unlink(enc_path)
     cmd = [self.gpg] + self.gpg_base_opts + self.gpg_quiet_opts + \
       self.gpg_fd_opts + [
-        '--no-default-keyring',
-        '--keyring', self.tmp_keyring,
-        '--always-trust',
-        '--armor',
-        '-r', key,
-        '--output', enc_path,
-        '-e', filename,
+          '--no-default-keyring',
+          '--keyring', self.tmp_keyring,
+          '--always-trust',
+          '--armor',
+          '-r', key,
+          '--output', enc_path,
+          '-e', filename,
       ]
-    debug(cmd)
+    logcmd(cmd)
     gpg = subprocess.Popen(cmd, stdin=subprocess.PIPE,
                            stdout=subprocess.PIPE,
                            stderr=self.null,
@@ -388,7 +424,7 @@ class PiusSigner(object):
         debug('Got GPG_KEY_EXP')
         continue
       else:
-        raise EncryptionUnknownError, line
+        raise EncryptionUnknownError(line)
 
     gpg.wait()
     return enc_file
@@ -396,7 +432,7 @@ class PiusSigner(object):
   def _run_and_check_status(self, cmd):
     '''Helper function for running a gpg call that requires no input
     but that we want to make sure succeeded.'''
-    debug(cmd)
+    logcmd(cmd)
     gpg = subprocess.Popen(cmd, stdin=subprocess.PIPE,
                            stdout=self.null,
                            stderr=self.null,
@@ -412,11 +448,11 @@ class PiusSigner(object):
     if os.path.exists(path):
       os.unlink(path)
     cmd = [self.gpg] + self.gpg_base_opts + self.gpg_quiet_opts + [
-      '--no-default-keyring',
-      '--keyring', keyring,
-      '--armor',
-      '--output', path,
-      '--export',
+        '--no-default-keyring',
+        '--keyring', keyring,
+        '--armor',
+        '--output', path,
+        '--export',
     ] + keys
     self._run_and_check_status(cmd)
 
@@ -443,10 +479,10 @@ class PiusSigner(object):
     keyring.'''
     path = self._tmpfile_path('%s.asc' %  key)
     cmd = [self.gpg] + self.gpg_base_opts + self.gpg_quiet_opts + [
-      '--no-default-keyring',
-      '--keyring', self.tmp_keyring,
-      '--import-options', 'import-minimal',
-      '--import', path,
+        '--no-default-keyring',
+        '--keyring', self.tmp_keyring,
+        '--import-options', 'import-minimal',
+        '--import', path,
     ]
     self._run_and_check_status(cmd)
 
@@ -465,14 +501,14 @@ class PiusSigner(object):
   def sign_uid_expect(self, key, index, level):
     '''Sign a UID, using the expect stuff. Interactive mode.'''
     cmd = [self.gpg] + self.gpg_base_opts + [
-      '--no-default-keyring',
-      '--keyring', self.tmp_keyring,
-      '--default-cert-level', level,
-      '--no-ask-cert-level',
-      '--no-use-agent',
-      '--edit-key', key,
+        '--no-default-keyring',
+        '--keyring', self.tmp_keyring,
+        '--default-cert-level', level,
+        '--no-ask-cert-level',
+        '--no-use-agent',
+        '--edit-key', key,
     ] + self.policy_opts()
-    debug(' '.join(cmd))
+    logcmd(cmd)
     gpg = pexpect.spawn(' '.join((quote(arg) for arg in cmd)))
     gpg.setecho(False)
     gpg.expect('gpg> ')
@@ -523,13 +559,13 @@ class PiusSigner(object):
     # Note that if passphrase-fd is different from command-fd, nothing works.
     cmd = [self.gpg] + self.gpg_base_opts + self.gpg_quiet_opts + \
       self.gpg_fd_opts + keyring + [
-        '-u', self.signer,
+          '-u', self.signer,
       ] + agent + [
-        '--default-cert-level', level,
-        '--no-ask-cert-level',
-        '--edit-key', key,
+          '--default-cert-level', level,
+          '--no-ask-cert-level',
+          '--edit-key', key,
       ] + self.policy_opts()
-    debug(cmd)
+    logcmd(cmd)
     gpg = subprocess.Popen(cmd, stdin=subprocess.PIPE,
                            stdout=subprocess.PIPE,
                            stderr=self.null,
@@ -537,7 +573,8 @@ class PiusSigner(object):
 
     if self.mode == MODE_AGENT:
       # For some reason when using agent an initial enter is needed
-      gpg.stdin.write('\n')
+      if not self.gpg2:
+          gpg.stdin.write('\n')
     else:
       # For some unidentified reason you must send the passphrase
       # first, not when it asks for it.
@@ -611,13 +648,26 @@ class PiusSigner(object):
     while True:
       line = gpg.stdout.readline()
       debug('Got %s' % line)
+      # gpg1 + gpgagent1 reported BAD_PASSPHRASE for both the agent the wrong
+      # passphrase, and for canceling the prompt.
+      #
+      # gpg2.0 + gpgagent2.0 seems to do MISSING_PASSPHRASE and BAD_PASSPHRASE
+      # for the respective answers
+      #
+      # gpg2.1 + gpgagent2.1 seems to just do ERROR
+      if 'ERROR' in line:
+        print '  ERROR: Agent reported an error.'
+        raise AgentError
+      if 'MISSING_PASSPHRASE' in line:
+        print '  ERROR: Agent didn\'t provide passphrase to PGP.'
+        raise AgentError
       if 'BAD_PASSPHRASE' in line:
         if self.mode == MODE_AGENT:
           line = gpg.stdout.readline()
           debug('Got %s' % line)
           if 'USERID_HINT' in line:
             continue
-          print '  ERROR: Agent didn\'t provide passphrase to PGP.'
+          print '  ERROR: Agent reported the passphrase was incorrect.'
           raise AgentError
         else:
           print '  ERROR: GPG didn\'t accept the passphrase.'
@@ -625,11 +675,14 @@ class PiusSigner(object):
       if 'GOOD_PASSPHRASE' in line:
         break
       if PiusSigner.GPG_PROMPT in line:
+        if self.gpg2:
+          break;
         print '  ERROR: GPG didn\'t sign.'
         raise GpgUnknownError(line)
 
     debug('Saving key')
-    self.gpg_wait_for_string(gpg.stdout, PiusSigner.GPG_PROMPT)
+    if not self.gpg2:
+      self.gpg_wait_for_string(gpg.stdout, PiusSigner.GPG_PROMPT)
     gpg.stdin.write('save\n')
 
     gpg.wait()
@@ -638,21 +691,22 @@ class PiusSigner(object):
   def print_filenames(self, uids):
     '''Print the filenames we created for the user.'''
     print '  Signed UNencrypted keys: '
-    for index in range (1, len(uids)):
-      if uids[index]['status'] != 'r' and uids[index]['result']:
-        print '    %(id)s: %(file)s' % uids[index]
+    for uid in uids:
+      if uid['status'] != 'r' and uid['result']:
+        print '    %(id)s: %(file)s' % uid
     if self.encrypt_outfiles:
       print '  Signed encrypted keys: '
-      for index in range (1, len(uids)):
-        if uids[index]['status'] != 'r' and uids[index]['result']:
-          print '    %(id)s: %(enc_file)s' % uids[index]
+      for uid in uids:
+        if uid['status'] != 'r' and uid['result']:
+          print '    %(id)s: %(enc_file)s' % uids
 
   def sign_all_uids(self, key, level):
     '''The main function that signs all the UIDs on a given key.'''
     signed_any_uids = False
     uids = self.get_uids(key)
     print '  There %s %s UID%s on this key to sign' % (
-          ['is','are'][len(uids)!=1], len(uids), "s"[len(uids)==1:])
+        ['is', 'are'][len(uids) != 1], len(uids), "s"[len(uids) == 1:]
+    )
 
     # From the user key ring make a clean copy
     self.export_clean_key(key)
@@ -682,7 +736,7 @@ class PiusSigner(object):
                  ' I\'m bailing out!')
           sys.exit(1)
         except NoSelfKeyError:
-          print('\nWe don\'t have our own key, according to GnuPG.')
+          print '\nWe don\'t have our own key, according to GnuPG.'
           # No need to say anything else
           sys.exit(1)
       else:
@@ -701,7 +755,7 @@ class PiusSigner(object):
       if self.encrypt_outfiles:
         try:
           uid['enc_file'] = self._outfile_path(
-            self.encrypt_signed_uid(key, uid['file'])
+              self.encrypt_signed_uid(key, uid['file'])
           )
           sys.stdout.write(', encrypted')
         except EncryptionKeyError:
@@ -719,12 +773,12 @@ class PiusSigner(object):
       if self.mail:
         try:
           if uid['email'] == None:
-            print ('  WARNING: No email for %s, cannot send key.' % uid['id'])
+            print '  WARNING: No email for %s, cannot send key.' % uid['id']
             continue
           # this is a ugly. The mailer needs to be able to be able to call
           # encrypt_and_sign_file() to be able to generate the PGP/MIME file,
           # so we pass outselves, it can call it...
-          self.mailer.send_mail(self.signer, key, uid, self)
+          self.mailer.send_sig_mail(self.signer, key, uid, self)
           sys.stdout.write(', mailed')
         except MailSendError, msg:
           print ('\nThere was a problem talking to the mail server (%s): %s'
@@ -752,7 +806,7 @@ class PiusSigner(object):
     '''Import all the unsigned keys from keyring to main keyring.'''
     print 'Importing keyring...'
     cmd = [self.gpg] + self.gpg_base_opts + self.gpg_quiet_opts + [
-      '--import', self.keyring,
+        '--import', self.keyring,
     ]
     self._run_and_check_status(cmd)
 
@@ -765,18 +819,18 @@ class PiusSigner(object):
       agent = ['--use-agent']
     cmd = [self.gpg] + self.gpg_base_opts +  self.gpg_quiet_opts + \
       self.gpg_fd_opts + agent + [
-        '--no-default-keyring',
-        '--keyring', self.tmp_keyring,
-        '--no-options',
-        '--always-trust',
-        '-u', self.signer,
-        '-aes',
-        '-r', keyid,
-        '-r', self.signer,
-        '--output', outfile,
-        infile,
+          '--no-default-keyring',
+          '--keyring', self.tmp_keyring,
+          '--no-options',
+          '--always-trust',
+          '-u', self.signer,
+          '-aes',
+          '-r', keyid,
+          '-r', self.signer,
+          '--output', outfile,
+          infile,
       ]
-    debug(cmd)
+    logcmd(cmd)
     gpg = subprocess.Popen(cmd, stdin=subprocess.PIPE,
                            stdout=subprocess.PIPE,
                            stderr=self.null,
@@ -819,10 +873,10 @@ class PiusSigner(object):
         debug('Got skippable stuff')
         continue
       else:
-        raise EncryptionUnknownError, line
+        raise EncryptionUnknownError(line)
 
     retval = gpg.wait()
     if retval != 0:
-      raise EncryptionUnknownError, "Return code was %s" % retval
+      raise EncryptionUnknownError("Return code was %s" % retval)
 
 # END class PiusSigner
